;===============================================================;
;                                                               ;
; Alien Breed 5                                                 ;
; Object routines                                               ;
;                                                               ;
;===============================================================;

;------------------------------------------------
; checkObjectTrigger - check a tile to see if it's an object trigger
;   input:  HL => tile in level data
;   output: none
;------------------------------------------------
checkObjectTrigger:
        ld      b,NUM_OBJECT_TRIGGERS           ; B = # tile id's to check
        ld      de,objectTriggers               ; DE => list of tile id's to check
cotLoop:
        ld      a,(de)
        cp      (hl)                            ; check if tile in level data matches this entry
        jr      z,newObject                     ; if so, create new object
        inc     de
        inc     de                              ; DE => next tile id to check
        djnz    cotLoop
        ret

;------------------------------------------------
; newObject - create a new object
;   input:  HL => tile in level data that triggered the object
;           DE => entry in objectTriggers table
;   output: none
;------------------------------------------------
newObject:
        inc     de
        ld      a,(de)
        ld      c,a                             ; C = object type
        push    ix                              ; needs to be preserved for outside routine
        ld      ix,objectTable
        ld      b,NUM_OBJECTS
        ld      de,OBJECT_SIZE
findEmptyObjectEntry:
        ld      a,(ix)
        or      a
        jr      z,objectEntryFound
        add     ix,de
        djnz    findEmptyObjectEntry
        jr      endNewObject
objectEntryFound:
        ld      (ix),c                          ; save object type
        call    getExactXYFromHL
        ld      (ix+1),e
        ld      (ix+2),d                        ; save x coord
        ld      (ix+3),l
        ld      (ix+4),h                        ; save y coord
endNewObject:
        pop     ix                              ; for outside routine
        ret

;------------------------------------------------
; drawObjects - draw objects from objectTable
;   input:  none
;   output: none
;------------------------------------------------
drawObjects:
        ld      ix,objectTable
        ld      b,NUM_OBJECTS
drawObjectsLoop:
        push    bc
        ld      a,(ix)
        or      a
        jr      z,endDrawObjectsLoop            ; if empty entry, nothing to draw
        dec     a
#ifdef TI83PLUS
        add     a,a
        ld      e,a
        ld      d,0
#endif
#ifdef TI84PLUSCE
        ld      e,a
        ld      d,3
        mlt     de
#endif
        ld      hl,objectSpriteTable
        add     hl,de
        LDHLIND()
        ex      de,hl                           ; DE => first sprite in set for this object type
        call    getObjectFrame                  ; HL => sprite to draw    
        push    hl
        ld      de,(screenX)
        or      a
#ifdef TI84PLUSCE
        sbc     hl,hl
#endif
        ld      l,(ix+1)
        ld      h,(ix+2)
        sbc     hl,de
        ld      b,l                             ; B = x coord to draw at
        ld      de,(screenY)
        or      a
#ifdef TI84PLUSCE
        sbc     hl,hl
#endif
        ld      l,(ix+3)
        ld      h,(ix+4)
        sbc     hl,de
        ld      c,l                             ; C = y coord to draw at
        pop     hl                              ; HL => sprite    
        push    ix                              ; save objectTable ptr
#ifdef TI83PLUS
        call    putClippedSprite
#endif
#ifdef TI84PLUSCE
        ld      e,b
        ld      d,c
        call    drawClippedTransSprite
#endif
        pop     ix
endDrawObjectsLoop:
        pop     bc
        ld      de,OBJECT_SIZE
        add     ix,de
        djnz    drawObjectsLoop
        ret

;------------------------------------------------
; getObjectFrame - get pointer to sprite to use for an object
;   input:  IX => entry in objectTable
;           DE => first sprite in sprite set
;   output: HL => sprite to use
;------------------------------------------------
getObjectFrame:
        ld      a,(ix)                          ; A = object type
        dec     a
        jr      z,gofOneWayVentFan
        dec     a
        jr      z,gofOneWayVentFan
        dec     a
        jr      z,gofGarbageChute
        dec     a
        jr      z,gofOneWayVentFan
        dec     a
        jr      z,gofCompactor
        dec     a
        jr      z,gofCompactor
        ex      de,hl                           ; if no object type matched, just show the sprite that DE was pointing to
        ret
gofOneWayVentFan:
        ld      a,(frame)
        and     %00000110                       ; update object sprite every 2nd frame, and using an 4 sprite rotation for the object
        ld      l,a
#ifdef TI83PLUS
        ld      h,0
        add     hl,hl
        add     hl,hl
#endif
#ifdef TI84PLUSCE
        ld      h,128
        mlt     hl
#endif
        add     hl,de                           ; HL => sprite to use
        ret
gofGarbageChute:
        ld      a,(frame)
        srl     a                               ; A = {0-127}
        srl     a                               ; A = {0-63}
        srl     a                               ; A = {0-31}
        ld      hl,0
        cp      14
        jr      c,gofgcCalc
        jr      z,gofgc2
        cp      31
        jr      z,gofgc2
        cp      15
        jr      z,gofgc3
        cp      30
        jr      z,gofgc3
gofgc4:
#ifdef TI83PLUS
        ld      hl,8*3
#endif
#ifdef TI84PLUSCE
        ld      hl,256*3
#endif
        jr      gofgcCalc
gofgc3:
#ifdef TI83PLUS
        ld      hl,8*2
#endif
#ifdef TI84PLUSCE
        ld      hl,256*2
#endif
        jr      gofgcCalc
gofgc2:
#ifdef TI83PLUS
        ld      hl,8*1
#endif
#ifdef TI84PLUSCE
        ld      hl,256*1
#endif
gofgcCalc:
        add     hl,de
        ret
gofCompactor:
        ld      a,(frame)
        and     %00011100                       ; update object sprite every 4th frame, and using an 8 sprite rotation for the object
        ld      l,a
#ifdef TI83PLUS
        ld      h,0
        add     hl,hl
#endif
#ifdef TI84PLUSCE
        ld      h,64
        mlt     hl
#endif
        add     hl,de                           ; HL => sprite to use
        ret

;------------------------------------------------
; updateObjects - update objects/check if still on screen
;   input:  none
;   output: none
;------------------------------------------------
updateObjects:
        ld      ix,collideObject1
        call    loadScreenCollideObjectIXNoBorder ; will be used to check if objects are still on screen
        ld      ix,objectTable
        ld      b,NUM_OBJECTS
updateObjectsLoop:
        push    bc
        ld      a,(ix)
        or      a
        jr      z,endUpdateObjectsLoop          ; if empty entry, nothing to update
        call    loadObjectCollideObject2
        call    checkCollision
        jr      c,objectOnScreen                ; if colliding, the object is still on screen
        ld      (ix),0                          ; otherwise, clear the entry
objectOnScreen:
endUpdateObjectsLoop:
        pop     bc
        ld      de,OBJECT_SIZE
        add     ix,de
        djnz    updateObjectsLoop
        ret

.end
